@@ -1,8 +1,9 @@
 use base64::{decode, encode};
 use chrono::{DateTime, Duration, TimeZone, Utc};
+use http::HeaderMap;
 use serde;
 use serde::de::{Deserialize, Deserializer, Error as DeError};
-use serde::ser::Serializer;
+use serde::ser::{Error as SerError, SerializeMap, Serializer};
 use std;
 use std::collections::HashMap;
 
@@ -178,6 +179,77 @@ where
     Ok(Duration::minutes(minutes as i64))
 }
 
+/// Serialize a http::HeaderMap into a serde str => Vec<str> map
+pub(crate) fn serialize_multi_value_headers<S>(
+    headers: &HeaderMap,
+    serializer: S,
+) -> Result<S::Ok, S::Error>
+where
+    S: Serializer,
+{
+    let mut map = serializer.serialize_map(Some(headers.keys_len()))?;
+    for key in headers.keys() {
+        let mut map_values = Vec::new();
+        for value in headers.get_all(key) {
+            map_values.push(value.to_str().map_err(S::Error::custom)?)
+        }
+        map.serialize_entry(key.as_str(), &map_values)?;
+    }
+    map.end()
+}
+
+/// Serialize a http::HeaderMap into a serde str => str map
+pub(crate) fn serialize_headers<S>(headers: &HeaderMap, serializer: S) -> Result<S::Ok, S::Error>
+where
+    S: Serializer,
+{
+    let mut map = serializer.serialize_map(Some(headers.keys_len()))?;
+    for key in headers.keys() {
+        let map_value = headers[key].to_str().map_err(S::Error::custom)?;
+        map.serialize_entry(key.as_str(), map_value)?;
+    }
+    map.end()
+}
+
+pub mod http_method {
+    use http::Method;
+    use serde::de;
+    use serde::de::{Unexpected, Visitor};
+    use serde::{Deserializer, Serializer};
+    use std::fmt;
+
+    /// Implementation detail. Use derive annotations instead.
+    pub fn serialize<S: Serializer>(method: &Method, ser: S) -> Result<S::Ok, S::Error> {
+        ser.serialize_str(method.as_str())
+    }
+
+    struct MethodVisitor;
+    impl<'de> Visitor<'de> for MethodVisitor {
+        type Value = Method;
+
+        fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+            write!(formatter, "valid method name")
+        }
+
+        fn visit_str<E: de::Error>(self, val: &str) -> Result<Self::Value, E> {
+            if val.is_empty() {
+                Ok(Method::GET)
+            } else {
+                val.parse()
+                    .map_err(|_| de::Error::invalid_value(Unexpected::Str(val), &self))
+            }
+        }
+    }
+
+    /// Implementation detail.
+    pub fn deserialize<'de, D>(de: D) -> Result<Method, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        de.deserialize_str(MethodVisitor)
+    }
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
